DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    },
}
SECRET_KEY = "un33k"

# Static file finders in order of precedence
#######################################
STATICFILES_FINDERS = [
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    'django.contrib.staticfiles.finders.DefaultStorageFinder',
]

# Template file loads in order of precedence
#######################################
TEMPLATE_LOADERS = [
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
]

# Context processor
#######################################
TEMPLATE_CONTEXT_PROCESSORS = [
    'django.contrib.auth.context_processors.auth',
    'django.core.context_processors.debug',
    'django.core.context_processors.request',
    'django.contrib.messages.context_processors.messages',
]

# Installed Apps
#######################################
INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.humanize',
    'django.contrib.admin',

    'currencyware',
]

MIDDLEWARE_CLASSES = []
ROOT_URLCONF = 'currencyware.tests.urls'
SITE_ID = 1
TEST_RUNNER = 'django.test.runner.DiscoverRunner'

PRIORITY_CURRENCY_CODES = ['CAD']




from django.conf import settings

DEFAULT_CURRENY_LANGUAGE_CODE = getattr(settings, 'DEFAULT_CURRENY_LANGUAGE_CODE', 'en')

# https://justforex.com/education/currencies
ALL_CURRENCY_CODES = getattr(settings, 'ALL_CURRENCY_CODES', [
    'AED', 'AFN', 'ALL', 'AMD', 'AOA', 'ARS',
    'AUD', 'AWG', 'AZN', 'BAM', 'BBD', 'BDT',
    'BGN', 'BHD', 'BIF', 'BMD', 'BND', 'BOB',
    'BRL', 'BSD', 'BTN', 'BWP', 'BYR', 'BZD',
    'CAD', 'CDF', 'CHF', 'CLP', 'CNY', 'COP',
    'CRC', 'CUP', 'CVE', 'CZK', 'DJF', 'DKK',
    'DOP', 'DZD', 'EGP', 'ERN', 'ETB', 'EUR',
    'FJD', 'FKP', 'GBP', 'GEL', 'GHS', 'GIP',
    'GMD', 'GNF', 'GTQ', 'GYD', 'HKD', 'HNL',
    'HRK', 'HTG', 'HUF', 'IDR', 'ILS', 'INR',
    'IQD', 'IRR', 'ISK', 'JMD', 'JOD', 'JPY',
    'KES', 'KGS', 'KHR', 'KPW', 'KRW', 'KWD',
    'KYD', 'KZT', 'LAK', 'LBP', 'LKR', 'LRD',
    'LSL', 'LYD', 'MAD', 'MDL', 'MGA', 'MKD',
    'MMK', 'MNT', 'MOP', 'MRO', 'MUR', 'MVR',
    'MWK', 'MXN', 'MYR', 'MZN', 'NAD', 'NGN',
    'NIO', 'NOK', 'NPR', 'NZD', 'OMR', 'PAB',
    'PEN', 'PGK', 'PHP', 'PKR', 'PLN', 'PYG',
    'QAR', 'RON', 'RSD', 'RUB', 'RWF', 'SAR',
    'SBD', 'SCR', 'SDG', 'SEK', 'SGD', 'SHP',
    'SLL', 'SOS', 'SRD', 'STD', 'SYP', 'SZL',
    'THB', 'TJS', 'TMT', 'TND', 'TOP', 'TRY',
    'TTD', 'TWD', 'TZS', 'UAH', 'UGX', 'USD',
    'UYU', 'UZS', 'VEF', 'VND', 'VUV', 'WST',
    'XAF', 'XCD', 'XPF', 'YER', 'ZAR', 'ZMW',
])

PRIORITY_CURRENCY_CODES = getattr(settings, 'PRIORITY_CURRENCY_CODES', [
    'USD', 'CAD', 'EUR', 'AUD', 'GBP', 'HKD', 'JPY', 'CNY', 'CHF',
])

BASE_CURRENY_CODE = getattr(settings, 'BASE_CURRENY_CODE', 'USD')

# https://openexchangerates.org/api/historical/2001-02-16.json (historical)
# https://openexchangerates.org/api/latest.json (latest)
# GET params ?app_id=YOUR_APP_ID&base=GBP (USD is default base)
OPEN_EXCHANGE_RATES_URL = 'https://openexchangerates.org/api/'
OPEN_EXCHANGE_RATES_URL = getattr(settings, 'OPEN_EXCHANGE_RATES_URL', OPEN_EXCHANGE_RATES_URL)
OPEN_EXCHANGE_RATES_API_KEY = getattr(settings, 'OPEN_EXCHANGE_RATES_API_KEY', None)
OPEN_EXCHANGE_RATES_API_CALLS_PER_SECONDS = 1

GOOGLE_API_KEY = getattr(settings, 'GOOGLE_API_KEY', None)
GOOGLE_TRANSLATE_URL = 'https://www.googleapis.com/language/translate/v2'
# https://www.googleapis.com/language/translate/v2?key=xyz&q=Euro&source=en&target=zh-Hant


default_app_config = 'currencyware.apps.CurrencywareConfig'

__author__ = 'Val Neekman @ Neekware Inc. [@vneekman]'
__description__ = "A Django application to translate currency names"
__version__ = '0.1.3'



from django.apps import apps
from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class CurrencywareConfig(AppConfig):
    """
    Configuration entry point for the currencyware app
    """
    label = name = 'currencyware'
    verbose_name = _("currencyware app")



import locale
import functools

from django.utils import translation
from django.utils.translation import ugettext as _

from . import defaults as defs
from .utils import get_cache_key, get_from_cache, set_to_cache


def get_display(code):
    # Note: admin:skip
    display = _('ISO_4217.' + code)
    return display


def get_currencies(codes):
    """ Returns a list of (code, translation) tuples for codes  """
    currencies = [(code, get_display(code)) for code in codes]
    return currencies


def get_all_currencies(codes=defs.ALL_CURRENCY_CODES):
    """ Returns a list of (code, translation) tuples for all currency codes  """
    key = get_cache_key(codes)
    currencies = get_from_cache(key)
    if currencies:
        return currencies
    currencies = get_currencies(codes)
    set_to_cache(key, currencies)
    return currencies


def get_all_currencies_sorted(codes=defs.ALL_CURRENCY_CODES):
    """ Returns a list of (code, translation) tuples for all currency codes  """
    key = get_cache_key(codes, True)
    currencies = get_from_cache(key)
    if currencies:
        return currencies
    currencies = sorted(
        get_currencies(codes),
        key=functools.cmp_to_key(lambda a, b: locale.strcoll(a[1], b[1]))
    )
    set_to_cache(key, currencies)
    return currencies


def get_all_currencies_prioritized(
        priority_codes=defs.PRIORITY_CURRENCY_CODES,
        codes=defs.ALL_CURRENCY_CODES
    ):
    """ Returns a sorted list of (code, translation) tuples for currency codes  """
    codes_subset = list(set(codes) - set(priority_codes))
    key = get_cache_key(priority_codes + codes_subset , True)
    currencies = get_from_cache(key)
    if currencies:
        return currencies
    currencies = get_from_cache(key)
    priority_currencies = get_all_currencies(priority_codes)
    currencies = get_all_currencies_sorted(codes_subset)
    combined_currencies = priority_currencies + currencies
    set_to_cache(key, combined_currencies)
    return combined_currencies



import time

from django.test import TestCase
from django.conf import settings
from django.utils import translation
from django.core.management import call_command

from currencyware.currency import get_all_currencies
from currencyware.currency import get_all_currencies_sorted
from currencyware.currency import get_all_currencies_prioritized
from currencyware.currency import get_display
from currencyware import defaults as defs


class TestCountryCase(TestCase):
    """
    Country Test
    """
    def setUp(self):
        # call_command('compilemessages')
        pass

    def test_xlate_display(self):
        name = get_display('AED')
        self.assertEquals(name, 'UAE Dirham')

    def test_xlate_priority(self):
        translation.activate('zh-hant')
        name = get_display('AED')
        self.assertEquals(name, '阿拉伯聯合大公國迪拉姆')

    def test_xlate_en_unsorted(self):
        translation.activate('en')
        currencies = get_all_currencies()
        self.assertEquals(currencies[0][1], 'UAE Dirham')

    def test_xlate_en_sorted(self):
        translation.activate('en')
        currencies = get_all_currencies_sorted()
        self.assertEquals(currencies[0][1], 'Afghani')

    def test_xlate_en_prioritized(self):
        translation.activate('en')
        currencies = get_all_currencies_prioritized()
        self.assertEquals(currencies[0][1], 'Canadian Dollar')

    def test_xlate_fa_prioritized(self):
        translation.activate('fr')
        currencies = get_all_currencies_prioritized()
        self.assertEquals(currencies[0][1], 'Dollar canadien')


import re
import os
import sys
import json
import codecs
import logging
import requests
from datetime import datetime
import polib

from django.conf import settings
from django.utils import timezone
from django.utils import translation
from django.core.management.base import BaseCommand
from django.core.management.base import CommandError

from toolware.utils.generic import get_days_ago

from ...currency import get_display
from ...models import Rate, Currency
from ... import defaults as defs

log = logging.getLogger(__name__)

ISO_MAP = {
  'he': 'iw',
  'zh-hans': 'zh-cn'
}


class Command(BaseCommand):
    # Experimental 
    help = "Fetches live rates from Google translation api"
    path = os.path.abspath(os.path.join(os.path.realpath(__file__), '../../../', 'locale'))
    from_locale = 'en'
    GOOGLE_TRANSLATE_URL = defs.GOOGLE_TRANSLATE_URL
    GOOGLE_API_KEY = defs.GOOGLE_API_KEY
    
    def add_arguments(self, parser):
        parser.add_argument(
            '-l',
            '--locale',
            dest='locale',
            help='Translate locale of for language.'
        )

        parser.add_argument(
            '-o',
            '--overwrite',
            dest='overwrite',
            default=False,
            help='Overwrite translated items.'
        )

    def handle(self, *args, **options):
        self.verbosity = options['verbosity']
        locale = options.get('locale')
        self.overwrite = options.get('overwrite')
        if not locale:
            self.print_help("", subcommand='currency')
            return

        locales = locale.split(' ')
        if locales:
            if self.verbosity > 2:
                self.stdout.write('Preparing to fetch translations for locale {} ...'.format(locale))

            source_path = os.path.join(self.path, self.from_locale, 'LC_MESSAGES', 'django.po')
            source_po = polib.pofile(source_path)
            source_pairs = self.get_source_msg_pairs(source_po)

            for target in locales:
                self.process_target(source_pairs, target)

        
    def process_target(self, source_pairs, target_locale):
        target_path = os.path.join(self.path, target_locale, 'LC_MESSAGES', 'django.po')
        if not os.path.isfile(target_path):
            self.stdout.write('File not found {} ...'.format(target_path))
            return

        target_po = polib.pofile(target_path)
        target_pairs = self.get_source_msg_pairs(target_po)

        self.stdout.write('Processing {} ...'.format(target_locale))
        for index, pair in enumerate(source_pairs):
            text = pair[1]
            success, value = self.xlate(text, target_locale)
            if not success:
                break
            if self.verbosity > 1:
                print(text, value, target_locale)
            try:
                target_po[index].msgstr = self.post_translate_cleanup(value)
            except IndexError as err:
                continue
            target_po.save()

    def get_source_msg_pairs(self, po):
        """
        Returns translated msg (id:str) from a po file
        """
        msg_pairs = []
        for index, item in enumerate(po):
            msg_pairs.append((item.msgid, self.pre_translate_cleanup(item.msgstr)))
        return msg_pairs

    def should_translate(self, item):
        if item.obsolete:
            return False
        if item.translated() and self.overwrite:
            return True
    
    def pre_translate_cleanup(self, msgstr):
        """
        Best attempt to ensure google doesn't translate the following
        """
        return msgstr
    
    def post_translate_cleanup(self, msgstr):
        """ Restore reserved placeholders """
        return msgstr

    def xlate(self, text, target, source='en'):
        if target in ISO_MAP:
            target = ISO_MAP[target]
        resp = requests.get(
            self.GOOGLE_TRANSLATE_URL,
            params={
                'key': self.GOOGLE_API_KEY,
                'q': text,
                'source': source,
                'target': target
            }
        )

        if resp.status_code != requests.codes.ok:
            self.stdout.write('Failed to fetch rates ...')
            self.stdout.write(resp.text)
            return (False, None)

        return (True, resp.json()['data']['translations'][0]['translatedText'])



import sys
import json
import codecs
import logging
import requests
from datetime import datetime

from django.conf import settings
from django.utils import timezone
from django.utils import translation
from django.core.management.base import BaseCommand
from django.core.management.base import CommandError

from toolware.utils.generic import get_days_ago

from ...currency import get_display
from ...models import Rate, Currency
from ... import defaults as defs
from ... import utils as util

log = logging.getLogger(__name__)


class Command(BaseCommand):
    # Translators: admin
    help = "Fetches live rates from open exchange rates api"
    MIN_PURGE_DAYS = 2
    MAX_CHECKPOINT_DAYS = 30
    OXR_URL = defs.OPEN_EXCHANGE_RATES_URL
    OXR_KEY = defs.OPEN_EXCHANGE_RATES_API_KEY
    LANGUAGE_CODE = getattr(settings, 'LANGUAGE_CODE', 'en')
    
    def add_arguments(self, parser):
        parser.add_argument(
            '--purge',
            action='store',
            dest='purge_days',
            type=int,
            help='Remove rates older than x days'
        )

        parser.add_argument(
            '-c',
            '--checkpoint',
            action='store',
            dest='checkpoint_days',
            type=int,
            help='Restore missing rates within the last x days'
        )

        parser.add_argument(
            '-f',
            '--fetch',
            action='store_true',
            dest='fetch',
            default=False,
            help='Fetch and load all rates to db'
        )

    def handle(self, *args, **options):
        translation.activate(self.LANGUAGE_CODE)
        self.verbosity = options['verbosity']
        purge_days = options['purge_days']
        checkpoint_days = options['checkpoint_days']
        fetch = options['fetch']

        if not (purge_days or checkpoint_days or fetch):
            self.print_help("", subcommand='rate')
            return

        if purge_days:
            if purge_days < self.MIN_PURGE_DAYS:
                # purge only after `MIN_PURGE_DAYS` days old (compensate for UTC)
                self.stdout.write('Purge only possible for data >= {} days'.format(self.MIN_PURGE_DAYS))
                return
            self.purge(purge_days)

        if checkpoint_days:
            confirm = 'yes'
            if checkpoint_days > self.MAX_CHECKPOINT_DAYS:
                # restore rates within the last `MAX_CHECKPOINT_DAYS` (mind UTC)
                self.stdout.write('Restoring rates will hit the api {} times'.format(checkpoint_days))
                confirm = input('Are you sure? [yes | no]: ')
            if confirm == 'yes':
                self.checkpoint(checkpoint_days)

        if fetch:
            self.fetch()

    def fetch(self):

        if self.verbosity > 2:
            self.stdout.write('Preparing to fetch rates ...')
        latest_rate_url = requests.compat.urljoin(self.OXR_URL, 'latest.json')
        resp = self.fetch_url(latest_rate_url, params={'app_id': self.OXR_KEY, 'base': defs.BASE_CURRENY_CODE})
        if resp.status_code != requests.codes.ok:
            self.stdout.write('Failed to fetch rates ...')
            self.stdout.write(resp.text)
            return

        data = resp.json()
        create_count, update_count = self.add_or_update_rates(data)
        self.stdout.write('Created {count} currenies'.format(count=create_count))
        self.stdout.write('Updated {count} currenies'.format(count=update_count))

    def add_or_update_rates(self, data):
        create_count, update_count = 0, 0
        updated = datetime.fromtimestamp(data['timestamp'], tz=timezone.utc)
        for code, rate in data['rates'].items():
            created = False
            defaults = {
                'code': code,
                'date': updated,
                'rate': rate,
                'name': get_display(code)
            }
            instance, created = Rate.objects.get_or_create_unique(defaults, ['code', 'date'])
            if created:
                create_count += 1
            else:
                update_count += 1
            
            if not instance and self.verbosity >=3:
                sys.stdout.write('Failed to create rate for ({code})\n'.format(code=code))
        return create_count, update_count

    def checkpoint(self, days):
        did_checkpoint = False
        create_count, update_count = 0, 0
        for ago in range(1, days+1):
            exact_date = get_days_ago(ago)
            historical_uri = 'historical/{}.json'.format(exact_date.strftime('%Y-%m-%d'))
            historical_rate_url = requests.compat.urljoin(self.OXR_URL, historical_uri)
            resp = self.fetch_url(historical_rate_url, params={'app_id': self.OXR_KEY, 'base': defs.BASE_CURRENY_CODE})
            if resp.status_code != requests.codes.ok:
                if self.verbosity >= 2:
                    self.stdout.write('Failed to fetch historical rates for {} ...').format(historical_date)
                    self.stdout.write(resp.text)
                continue

            Rate.objects.filter(
                date__year=exact_date.year,
                date__month=exact_date.month,
                date__day=exact_date.day).delete()
        
            data = resp.json()
            create_count, update_count = self.add_or_update_rates(data)

        self.stdout.write('Checkpointed {count} currenies'.format(count=create_count))


    def purge(self, days):
        did_purge = False
        for code in defs.ALL_CURRENCY_CODES:
            for ago in range(self.MIN_PURGE_DAYS, days+1):
                exact_date = get_days_ago(ago)
                try:
                    latest_for_day = Rate.objects.filter(
                        code=code,
                        date__year=exact_date.year,
                        date__month=exact_date.month,
                        date__day=exact_date.day).latest('date')
                except Rate.DoesNotExist as err:
                    continue
                
                old_rates = Rate.objects.filter(
                    code=code,
                    date__year=exact_date.year,
                    date__month=exact_date.month,
                    date__day=exact_date.day).exclude(date=latest_for_day.date)
            
                if old_rates:
                    did_purge = True
                    old_rates.delete()
                    if self.verbosity >= 2:
                        self.stdout.write('Purged rates from ({}) days ago.'.format(ago))

        if did_purge:
            self.stdout.write('Purged rates older than ({}) ago from db.'.format(days))
        else:
            self.stdout.write('Nothing to purge')

    @util.rate_limited(defs.OPEN_EXCHANGE_RATES_API_CALLS_PER_SECONDS)
    def fetch_url(self, url, params):
        resp = requests.get(url, params=params)
        return resp


import os
import sys
import codecs
import logging
import json

from django.conf import settings
from django.core.management.base import BaseCommand
from django.core.management.base import CommandError
from django.utils.translation import activate

from ...models import Currency, Rate
from ...currency import get_display
from ... import defaults as defs

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    # Note: admin:skip
    help = 'Load currency data'
    path = os.path.abspath(os.path.join(os.path.realpath(__file__), '../../../', 'currency.json'))

    def add_arguments(self, parser):
        parser.add_argument(
            '-p',
            '--path',
            dest='path',
            default=self.path,
            help='Path to a json currency file.'
        )

        parser.add_argument(
            '--flush',
            dest='flush',
            default=False,
            action='store_true',
            help='Delete all existing currencies in db'
        )

        parser.add_argument(
            '-l',
            '--load',
            dest='load',
            action='store_true',
            default=False,
            help='Load currencies from data file'
        )

        parser.add_argument(
            '-o',
            '--overwrite',
            dest='overwrite',
            action='store_true',
            default=False,
            help='Overwrite currencies if already found in db'
        )

    def handle(self, *args, **options):
        self.verbosity = options['verbosity']
        path = options['path']
        overwrite = options['overwrite']
        flush = options['flush']
        load = options['load']
        
        if not (flush or load):
            self.print_help("", subcommand='currency')
            return

        if flush:
            self.flush()

        if load:
            self.load(path, overwrite)
    
    def flush(self):
        self.stdout.write('You are about to delete all currencies from db')
        confirm = input('Are you sure? [yes/no]: ')
        if confirm == 'yes':
            Currency.objects.all().delete()
            self.stdout.write('Currencies deleted from db.')

    def load(self, path, overwrite):
            
        if not os.path.isfile(path):
            self.stdout.write('No currency file found at path')
            self.stdout.write(path)
            self.print_help("", subcommand='currency')
            return

        activate(defs.DEFAULT_CURRENY_LANGUAGE_CODE)

        if self.verbosity > 2:
            self.stdout.write('Preparing currency file ...')

        fp = codecs.open(path, encoding='utf-8')
        self.data = json.load(fp)

        new_count, update_count = 0, 0
        for curr in self.data:
            created = False
            defaults = {
                'code': curr.get('code'),
                'name': get_display(curr.get('code')),
                'number': curr.get('number', 0),
                'symbol': curr.get('symbol', ''),
                'unit': curr.get('unit', 2),
                'country': ' '.join(curr.get('country', [])),
            }
            if overwrite:
                instance, created = Currency.objects.get_or_create_unique(defaults, ['code'])
            else:
                instance = Currency.objects.get_unique_or_none(code=defaults['code'])
                if not instance:
                    instance, created = Currency.objects.get_or_create_unique(defaults, ['code'])

            if created:
                new_count += 1
            elif overwrite:
                update_count += 1
        
        self.stdout.write('Created {count} currenies'.format(count=new_count))
        self.stdout.write('Updated {count} currenies'.format(count=update_count))




from django.db import models
from django.core.cache import cache
from django.utils.translation import ugettext as _
from django.core.validators import MaxValueValidator, MinValueValidator

from toolware.utils.query import CaseInsensitiveManager, CaseInsensitiveUniqueManager
from .currency import get_all_currencies_prioritized, get_display
from . import defaults as defs


class Currency(models.Model):
    code = models.CharField(
        # Note: admin:skip
        _('Code'),
        max_length=3,
        primary_key=True,
        null=False,
        blank=False,
        # Note: admin:skip
        help_text=_('Currency code')
    )

    name = models.CharField(
        # Note: admin:skip
        _('Name'),
        max_length=60,
        null=True,
        blank=True,
        # Note: admin:skip
        help_text=_('Curreny name (english)'),
    )
    
    number = models.CharField(
        # Note: admin:skip
        _('Number'),
        max_length=3,
        null=True,
        blank=True,
        # Note: admin:skip
        help_text=_('Numeric code'),
    )

    unit = models.IntegerField(
        # Note: admin:skip
        _('Unit'),
        null=True,
        blank=True,
        # Note: admin:skip
        help_text=_('Currency unit')
    )

    symbol = models.CharField(
        # Note: admin:skip
        _('Symbol'),
        max_length=10,
        null=True,
        blank=True,
        # Note: admin:skip
        help_text=_('Currency symbol'),
    )

    country = models.CharField(
        # Note: admin:skip
        _('Country'),
        max_length=255,
        null=True,
        blank=True,
        # Note: admin:skip
        help_text=_('Primary currency in these countries'),
    ) 

    # ########## Add new fields above this line #############
    objects = CaseInsensitiveUniqueManager()

    CASE_INSENSITIVE_FIELDS = ['code', 'name']

    @property
    def native_name(self):
        name = get_display(self.code)
        if self.code in name:
            name = self.name
        return name

    def __str__(self):
        return '{} ({})'.format(self.native_name, self.code)

    class Meta:
        # Note: admin:skip
        verbose_name=_('Currency')
        # Note: admin:skip
        verbose_name_plural=_('Currencies')


class Rate(models.Model):
    code = models.CharField(
        # Note: admin:skip
        _('Code'),
        max_length=3,
        null=False,
        # Note: admin:skip
        help_text=_('Currency code'),
    )
    name = models.CharField(
        # Note: admin:skip
        _('Name'),
        max_length=100,
        null=True,
        blank=True,
        # Note: admin:skip
        help_text=_('Curreny name (english)'),
    )
    rate = models.FloatField(
        # Note: admin:skip
        _('Rate'),
        null=False,
        blank=False,
        default=0.0,
        # Note: admin:skip
        help_text=_('Currency forex rate'),
    )

    date = models.DateTimeField(
        # Note: admin:skip
        _('Date'),
        null=False,
        blank=False,
        # Note: admin:skip
        help_text=_("Rate's date"),
    )

    # ########## Add new fields above this line #############
    objects = CaseInsensitiveUniqueManager()

    CASE_INSENSITIVE_FIELDS = ['code', 'name',]

    @property
    def native_name(self):
        name = get_display(self.code)
        if self.code in name:
            name = self.name
        return name

    def __str__(self):
        return '{} ({})'.format(self.native_name, self.code)
        
    class Meta:
        # Note: admin:skip
        verbose_name=_('Rate')
        # Note: admin:skip
        verbose_name_plural=_('Rates')


    @classmethod
    def get_rate(cls, source, target=defs.BASE_CURRENY_CODE):
        """
        Returns value in source to target.
        Example: CAD (source) to EUR (target)
        Rate.get_rate('CAD', 'EUR') => 0.6501663706682155
        50 CAD = 50 * 0.6501663706682155 = 32.50831853341077 EUR
        Default base currency is USD and can be changed via BASE_CURRENY_CODE
        in settings.py
        """
        try:
            source_rate = cls.objects.filter(code=source).latest('date')
            if target == defs.BASE_CURRENY_CODE:
                return 1 / source_rate.rate
            target_rate = cls.objects.filter(code=target).latest('date')
            if source_rate and target_rate:
                rate = target_rate.rate / source_rate.rate
        except cls.DoesNotExist as err:
            rate = 0.0
        return rate


# Generated by Django 2.0.8 on 2018-08-18 11:09

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Currency',
            fields=[
                ('code', models.CharField(help_text='Currency code', max_length=3, primary_key=True, serialize=False, verbose_name='Code')),
                ('name', models.CharField(blank=True, help_text='Curreny name (english)', max_length=60, null=True, verbose_name='Name')),
                ('number', models.CharField(blank=True, help_text='Numeric code', max_length=3, null=True, verbose_name='Number')),
                ('unit', models.IntegerField(blank=True, help_text='Currency unit', null=True, verbose_name='Unit')),
                ('symbol', models.CharField(blank=True, help_text='Currency symbol', max_length=10, null=True, verbose_name='Symbol')),
                ('country', models.CharField(blank=True, help_text='Primary currency in these countries', max_length=255, null=True, verbose_name='Country')),
            ],
            options={
                'verbose_name': 'Currency',
                'verbose_name_plural': 'Currencies',
            },
        ),
        migrations.CreateModel(
            name='Rate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(help_text='Currency code', max_length=3, verbose_name='Code')),
                ('name', models.CharField(blank=True, help_text='Curreny name (english)', max_length=100, null=True, verbose_name='Name')),
                ('rate', models.FloatField(default=0.0, help_text='Currency forex rate', verbose_name='Rate')),
                ('date', models.DateTimeField(help_text="Rate's date", verbose_name='Date')),
            ],
            options={
                'verbose_name': 'Rate',
                'verbose_name_plural': 'Rates',
            },
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import re
import os
import sys
import codecs
from setuptools import setup, find_packages


name = 'django-currencyware'
package = 'currencyware'
description = "A Django utility application that provides translated currency name services"
url = 'https://github.com/un33k/django-currencyware'
author = 'Val Neekman'
author_email = 'info@neekware.com'
license = 'MIT'
install_requires = ['django-toolware>=1.0.5']

classifiers = [
    'Development Status :: 4 - Beta',
    'Environment :: Web Environment',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.2',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Topic :: Utilities'

]


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = codecs.open(os.path.join(package, '__init__.py'), encoding='utf-8').read()
    return re.search("^__version__ = ['\"]([^'\"]+)['\"]", init_py, re.MULTILINE).group(1)


if sys.argv[-1] == 'build':
    os.system("python setup.py sdist bdist_wheel")

if sys.argv[-1] == 'publish':
    os.system("python setup.py sdist upload")
    args = {'version': get_version(package)}
    print("You probably want to also tag the version now:")
    print("  git tag -a %(version)s -m 'version %(version)s' && git push --tags" % args)
    sys.exit()

EXCLUDE_FROM_PACKAGES = []

setup(
    name=name,
    version=get_version(package),
    url=url,
    license=license,
    description=description,
    long_description=description,
    author=author,
    author_email=author_email,
    packages=find_packages(exclude=EXCLUDE_FROM_PACKAGES),
    include_package_data=True,
    install_requires=install_requires,
    classifiers=classifiers,
)



from django.contrib import admin

from .models import Currency, Rate


@admin.register(Currency)
class CurrencyAdmin(admin.ModelAdmin):
    list_display = ('code', 'name', 'symbol', 'number', 'unit', 'country', )
    search_fields = ('code', 'name', 'number',)
    ordering = ['code', 'name']


@admin.register(Rate)
class RateAdmin(admin.ModelAdmin):
    list_display = ('code', 'rate', 'date', 'name',)
    search_fields = ('code', 'name',)
    ordering = ['code', 'name',]



from django.contrib import admin
from django.conf.urls import url, include
from django.contrib.sites.models import Site

admin.autodiscover()
Site.objects.get_or_create(name='unittest', domain='unitest.org')

urlpatterns = [
    url(r'^admin/', admin.site.urls),
]